<?php

/**
 * @file
 * The Yammer API class.
 *
 * Here we directly utilize OAuth for authenticating against Yammer, and
 * implement some of the basic API calls.
 *
 * @see https://www.yammer.com/api_doc.html
 */
 
if (class_exists('OAuthException') != TRUE) {
  module_load_include('php', 'yammer_feeds', 'api/OAuth');
}

/**
 * The class contains functions those will be used to connect with yammer
 * via OAuth Authentication
 */
class YammerFeedYammerAPI {
  public $sig_method;
  public $consumer;
  public $request_tok;
  public $access_request;
  public $callback;
  public $domain;
  public $access_token;
  public $access_secret;
  public $message_count;

  /**
   * Constructor
   *
   * @param $key 
   *   The consumer key of the API
   * @param $secret
   *   The consumer secret of the application
   */
  function __construct($key, $secret) {
    $this->domain = "https://www.yammer.com/oauth";
    $this->callback = 'http://' . $_SERVER['HTTP_HOST'];
    $this->host = "https://www.yammer.com/api/v1/messages.json";
    $this->sig_method = new OAuthSignatureMethod_PLAINTEXT();
    $this->consumer = new OAuthConsumer($key, $secret, NULL);
    $this->request_tok = OAuthRequest::from_consumer_and_token($this->consumer, NULL, "POST", $this->domain . "/request_token");
    $message_state = array();
    $message_count = 0;
  }

  /**
   * Get authorization tokens used to create authorization URLs. Calling
   * function is responsible for error checking.
   *
   * @return array
   *   - string oauth_token
   *   - string oauth_token_secret
   *   - bool   oauth_callback_confirmed
   *   or error message
   */
  function getAuthorizationTokens() {
    // Set URL where we will read Authorization Tokens after Authorization.
    $this->request_tok->sign_request($this->sig_method, $this->consumer, NULL);
    
    // Settings for CURL options.
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_POSTFIELDS, '');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array($this->request_tok->to_header()));
    curl_setopt($ch, CURLOPT_URL, $this->domain . "/request_token");
    curl_setopt($ch, CURLOPT_POST, 1);
    $output = curl_exec($ch);
    curl_close($ch);
    parse_str($output, $oauth);
    return $oauth;
  }

  /**
   * Redirect user to Yammer application's authorize request url.
   *
   * @param $token
   *   oauth_token found in the callback URL
   */
  function authorizeRequest($token) {
    $auth_url = "https://www.yammer.com/oauth/authorize?oauth_token=" . $token . "&oauth_callback=" . urlencode($this->callback);
    Header("Location: $auth_url");
  }

  /**
   * Get access token of the user that will be used to pull messages from 
   * yammer network.
   *
   * @param $key
   *   The consumer key of the API
   * @param $secret
   *   The consumer secret of the application
   * @param $callback
   *   The path where application is supposed to return after authorization
   * @param $token
   *   oauth_token found in the callback URL
   * @param $tokensecret
   *   The secret token that was found in $oauth from getAuthorizationTokens()
   * @param $verifier
   *   oauth_verifier found in the call back URL
   *
   * @return array
   *   - string oauth_token
   *   - string oauth_token_secret
   *   - bool   oauth_callback_confirmed
   *   or error message
   */
  function getAccessToken($key, $secret, $callback, $token, $tokensecret, $verifier) {
    $req_token = new OAuthConsumer($token, $tokensecret, 1);
    $acc_req = OAuthRequest::from_consumer_and_token($this->consumer, $req_token, "POST", $this->domain . '/access_token');
    $acc_req->set_parameter("oauth_verifier", $verifier); # need the verifier too!
    $acc_req->sign_request($this->sig_method, $this->consumer, $req_token);
    
    // Curl Options settings.
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_POSTFIELDS, ''); //New Line
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array($acc_req->to_header()));
    curl_setopt($ch, CURLOPT_URL, $this->domain . "/access_token");
    curl_setopt($ch, CURLOPT_POST, 1);
    $output = curl_exec($ch);
    curl_close($ch);
    parse_str($output, $oauth);
    return $oauth;
  }

  /**
   * Get user information of a yammer user, The request type is GET.
   *
   * @param $token
   *   The consumer key of the API
   * @param $secret
   *   The consumer secret of the API
   * @param $user_id: 
   *   User Id
   */
  function getUserInfo($token, $secret, $user_id) {
    static $user_info;
    if (!isset($user_info[$user_id])) {
      $host = "https://www.yammer.com/api/v1/users/" . $user_id . ".json";
      $args = array();
      $req_token = new OAuthConsumer($token, $secret, 1);
      $profile_req = OAuthRequest::from_consumer_and_token($this->consumer, $req_token, 'GET', $host);
      $profile_req->sign_request($this->sig_method, $this->consumer, $req_token);
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array($profile_req->to_header()));
      curl_setopt($ch, CURLOPT_URL, $host);
      curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
      curl_setopt($ch, CURLOPT_POSTFIELDS, $args);
      $output = curl_exec($ch);
      curl_close($ch);
      $user_info[$user_id] = json_decode($output);
    }
    return $user_info[$user_id];
  }

  /**
   * Retrieve the latest 20 messages.
   *
   * @param string $token
   *   The consumer key of the API
   * @param string $secret
   *   The consumer secret of the API
   * @param bool $older_than
   *   Optional parameter to get older messages.
   *
   * @return array
   *   Messages in an array format.
   */
  function getMessages($token, $secret, $older_than = FALSE) {
    $args = array();
    
    // Authorize
    $req_token = new OAuthConsumer($token, $secret, 1);
    $host = ($older_than == FALSE) ? $this->host : $this->host . '?older_than=' . $_SESSION['oldest_msg_id'];
    $profile_req = OAuthRequest::from_consumer_and_token($this->consumer, $req_token, "GET", $host);
    $profile_req->sign_request($this->sig_method, $this->consumer, $req_token);
    
    // curl
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array($profile_req->to_header()));
    curl_setopt($ch, CURLOPT_URL, $host);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
    curl_setopt($ch, CURLOPT_POSTFIELDS, $args);
    $output = curl_exec($ch);
    curl_close($ch);
    
    // Send back the JSON
    $json = json_decode($output);
    return $json->messages; 
  }

}
